#!/usr/bin/env python
# Delta Debugging with Fuzzers

import re
import random

tests = 1

def test(s):
    global tests
    print(repr(tests) + " " + repr(s) + " " + repr(len(s)))
    tests += 1
    
    # Simulate the program under test
    if len(s) > 5 and '.' in s and '{' in s:
        return "FAIL"
    else:
        return "PASS"

def ddmin(s):
    # assert test("") == "PASS"
    # assert test(s) == "FAIL"

    n = 2     # Initial granularity
    while len(s) >= 2:
        start = 0
        subset_length = len(s) // n   # Integer Division
        some_complement_is_failing = False

        while start < len(s):
            complement = s[:start] + s[start + subset_length:]

            if test(complement) == "FAIL":
                s = complement
                n = max(n - 1, 2)
                some_complement_is_failing = True
                break
                
            start += subset_length

        if not some_complement_is_failing:
            if n == len(s):
                break
            n = min(n * 2, len(s))

    return s

def fuzzer():
    string_length = int(random.random() * 1024)   # Strings up to 1024 characters long
    out = ""
    for i in range(0, string_length):
        out += chr(int(random.random() * 96 + 32)) # filled with ASCII 32..128
    return out

if __name__ == "__main__":

    # Produce some failing fuzz input
    s = ""
    while test(s) != "FAIL":
        s = fuzzer()
    assert (test(s) == "FAIL")

    # Minimize it
    s_min = ddmin(s)
    
    print(repr(s) + " => " + repr(s_min))
    