#!/usr/bin/env python

import sys, getopt
import random
import subprocess

def fuzzer(max_length, range_start, range_width):
    # Strings up to MAX_LENGTH characters long
    string_length = int(random.random() * max_length)
    out = ""
    for i in range(0, string_length):
        # filled with ASCII RANGE_START..RANGE_START+RANGE_WIDTH
        out += chr(int(range_start + random.random() * range_width))
    return out
    
def help():
    name = sys.argv[0]  # Program name
    print("Usage: " + name + " [-h] [-l MAX_LENGTH] [-s RANGE_START] [-w RANGE_WIDTH] [PROGRAM [ARGS]]")
    sys.exit(0)

def main():
    # Default values (local variables)
    max_length = 1024
    range_start = 32
    range_width = 96
    
    # Parse options using getopt.getopt()
    # Arguments to program are passed in sys.argv[] array
    # sys.argv[1:] is array slice starting with 1st arg
    # -h gives help
    # -l, -s, -w allow to set max_length, range_start, range_width
    opts, args = getopt.getopt(sys.argv[1:], "hl:s:w:")
    for opt, a in opts:
        if opt == "-h":
            help()
        elif opt == "-l":
            max_length = int(a)
        elif opt == "-s":
            range_start = int(a)
        elif opt == "-w":
            range_width = int(a)
        else:
            assert False, "unhandled option"
    
    # Create a fuzzed string
    fuzz = fuzzer(max_length, range_start, range_width)

    if len(args) == 0:
        # No arguments given - just print
        print(fuzz)
        sys.exit(0)
            
    # Start the program given in ARGS
    process = subprocess.Popen(args, 
        stdin=subprocess.PIPE,
        stdout=subprocess.PIPE,
        stderr=subprocess.PIPE)

    if sys.version_info >= (3, 0):
        # For Python 3:
        data = bytearray(fuzz, 'ascii')
    else:
        # For Python 2.x:
        data = fuzz

    # For debugging
    print(repr(data))
        
    # Send fuzz input to given process
    stdoutdata, stderrdata = process.communicate(data)
    
    # Wait for child to terminate
    process.wait()
    
    # Get return code (zero: ok, non-zero: failure)
    ret = process.returncode
    
    print(stdoutdata, stderrdata, ret)

if __name__ == "__main__":
    main()